package dao.impl;

import dao.StudentDAO;
import model.Student;
import util.DatabaseUtil;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * StudentDAO接口的实现类，提供对学生数据的访问操作
 * 实现了StudentDAO接口定义的所有方法，包括基础的CRUD操作和特定的查询方法
 * 使用JDBC与数据库进行交互，所有数据库操作都包含适当的异常处理
 */
public class StudentDAOImpl implements StudentDAO {
    
    /**
     * 插入一条学生记录
     * 将Student对象的信息保存到数据库中，并获取生成的主键ID
     * 
     * @param student 要插入的学生对象，包含学生的详细信息
     * @return 影响的行数，插入成功返回1，失败返回0
     */
    @Override
    public int insert(Student student) {
        String sql = "INSERT INTO students (student_id, name, class_name, contact, college, major, password) VALUES (?, ?, ?, ?, ?, ?, ?)";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS)) {
            
            // 设置预处理语句的参数
            stmt.setString(1, student.getStudentId());
            stmt.setString(2, student.getName());
            stmt.setString(3, student.getClassName());
            stmt.setString(4, student.getContact());
            stmt.setString(5, student.getCollege());
            stmt.setString(6, student.getMajor());
            stmt.setString(7, student.getPassword());
            
            int affectedRows = stmt.executeUpdate();
            if (affectedRows == 0) {
                return 0;
            }

            // 获取自动生成的主键
            try (ResultSet generatedKeys = stmt.getGeneratedKeys()) {
                if (generatedKeys.next()) {
                    student.setId(generatedKeys.getInt(1));
                }
            }
            return affectedRows;
        } catch (SQLException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 根据ID删除学生记录
     * 
     * @param id 要删除的学生记录的ID
     * @return 影响的行数，删除成功返回1，失败返回0
     */
    @Override
    public int deleteById(int id) {
        String sql = "DELETE FROM students WHERE id = ?";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql)) {
            
            stmt.setInt(1, id);
            return stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 更新学生信息
     * 根据学生对象的ID更新其他字段的值
     * 
     * @param student 包含更新信息的学生对象
     * @return 影响的行数，更新成功返回1，失败返回0
     */
    @Override
    public int update(Student student) {
        String sql = "UPDATE students SET student_id = ?, name = ?, class_name = ?, contact = ?, college = ?, major = ?, password = ? WHERE id = ?";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql)) {
            
            // 设置预处理语句的参数
            stmt.setString(1, student.getStudentId());
            stmt.setString(2, student.getName());
            stmt.setString(3, student.getClassName());
            stmt.setString(4, student.getContact());
            stmt.setString(5, student.getCollege());
            stmt.setString(6, student.getMajor());
            stmt.setString(7, student.getPassword());
            stmt.setInt(8, student.getId());
            
            return stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            return 0;
        }
    }

    /**
     * 根据ID查询学生信息
     * 
     * @param id 要查询的学生ID
     * @return 如果找到则返回学生对象，否则返回null
     */
    @Override
    public Student findById(int id) {
        String sql = "SELECT * FROM students WHERE id = ?";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql)) {
            
            stmt.setInt(1, id);
            ResultSet rs = stmt.executeQuery();
            
            if (rs.next()) {
                return mapResultSetToStudent(rs);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 查询所有学生记录
     * 
     * @return 包含所有学生对象的列表，如果没有记录则返回空列表
     */
    @Override
    public List<Student> findAll() {
        List<Student> students = new ArrayList<>();
        String sql = "SELECT * FROM students";
        try (Connection conn = DatabaseUtil.getConnection();
             Statement stmt = conn.createStatement();
             ResultSet rs = stmt.executeQuery(sql)) {
            
            while (rs.next()) {
                students.add(mapResultSetToStudent(rs));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return students;
    }

    /**
     * 根据学号查询学生信息
     * 
     * @param studentId 要查询的学号
     * @return 如果找到则返回学生对象，否则返回null
     */
    @Override
    public Student findByStudentId(String studentId) {
        String sql = "SELECT * FROM students WHERE student_id = ?";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql)) {
            
            stmt.setString(1, studentId);
            ResultSet rs = stmt.executeQuery();
            
            if (rs.next()) {
                return mapResultSetToStudent(rs);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 根据姓名模糊查询学生信息
     * 
     * @param name 要查询的学生姓名（支持模糊查询）
     * @return 符合条件的学生对象列表，如果没有匹配则返回空列表
     */
    @Override
    public List<Student> findByNameLike(String name) {
        List<Student> students = new ArrayList<>();
        String sql = "SELECT * FROM students WHERE name LIKE ?";
        try (Connection conn = DatabaseUtil.getConnection();
             PreparedStatement stmt = conn.prepareStatement(sql)) {
            
            stmt.setString(1, "%" + name + "%");
            ResultSet rs = stmt.executeQuery();
            
            while (rs.next()) {
                students.add(mapResultSetToStudent(rs));
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return students;
    }

    /**
     * 将ResultSet映射为Student对象
     * 从结果集中提取数据并创建Student实例
     * 
     * @param rs 包含学生数据的ResultSet对象
     * @return 映射后的Student对象
     * @throws SQLException 如果访问ResultSet时发生错误
     */
    private Student mapResultSetToStudent(ResultSet rs) throws SQLException {
        Student student = new Student();
        student.setId(rs.getInt("id"));
        student.setStudentId(rs.getString("student_id"));
        student.setName(rs.getString("name"));
        student.setClassName(rs.getString("class_name"));
        student.setContact(rs.getString("contact"));
        student.setCollege(rs.getString("college"));
        student.setMajor(rs.getString("major"));
        student.setPassword(rs.getString("password"));
        return student;
    }
}
